@@ -25,6 +25,7 @@ import logging
 
 from oslo_concurrency import lockutils
 from oslo_utils import excutils
+from oslo_utils import reflection
 import six
 
 from oslo_vmware._i18n import _, _LE, _LI, _LW
@@ -82,9 +83,9 @@ class RetryDecorator(object):
         self._sleep_time = 0
 
     def __call__(self, f):
+        func_name = reflection.get_callable_name(f)
 
         def _func(*args, **kwargs):
-            func_name = f.__name__
             result = None
             try:
                 if self._retry_count:
@@ -95,11 +96,11 @@ class RetryDecorator(object):
                 result = f(*args, **kwargs)
             except self._exceptions:
                 with excutils.save_and_reraise_exception() as ctxt:
-                    LOG.warn(_LW("Exception which is in the suggested list of "
-                                 "exceptions occurred while invoking function:"
-                                 " %s."),
-                             func_name,
-                             exc_info=True)
+                    LOG.warning(_LW("Exception which is in the suggested list "
+                                    "of exceptions occurred while invoking "
+                                    "function: %s."),
+                                func_name,
+                                exc_info=True)
                     if (self._max_retry_count != -1 and
                             self._retry_count >= self._max_retry_count):
                         LOG.error(_LE("Cannot retry upon suggested exception "
@@ -118,7 +119,7 @@ class RetryDecorator(object):
         def func(*args, **kwargs):
             loop = loopingcall.DynamicLoopingCall(_func, *args, **kwargs)
             evt = loop.start(periodic_interval_max=self._max_sleep_time)
-            LOG.debug("Waiting for function %s to return.", f.__name__)
+            LOG.debug("Waiting for function %s to return.", func_name)
             return evt.wait()
 
         return func
@@ -138,7 +139,8 @@ class VMwareAPISession(object):
     def __init__(self, host, server_username, server_password,
                  api_retry_count, task_poll_interval, scheme='https',
                  create_session=True, wsdl_loc=None, pbm_wsdl_loc=None,
-                 port=443, cacert=None, insecure=True):
+                 port=443, cacert=None, insecure=True, pool_size=10,
+                 connection_timeout=None):
         """Initializes the API session with given parameters.
 
         :param host: ESX/VC server IP address or host name
@@ -158,6 +160,10 @@ class VMwareAPISession(object):
                        TLS (https) server certificate.
         :param insecure: Verify HTTPS connections using system certificates,
                          used only if cacert is not specified
+        :param pool_size: Maximum number of connections in http
+                          connection pool
+        :param connection_timeout: Maximum time in seconds to wait for peer to
+                                   respond.
         :raises: VimException, VimFaultException, VimAttributeException,
                  VimSessionOverLoadException
         """
@@ -176,6 +182,8 @@ class VMwareAPISession(object):
         self._pbm = None
         self._cacert = cacert
         self._insecure = insecure
+        self._pool_size = pool_size
+        self._connection_timeout = connection_timeout
         if create_session:
             self._create_session()
 
@@ -192,7 +200,9 @@ class VMwareAPISession(object):
                                 port=self._port,
                                 wsdl_url=self._vim_wsdl_loc,
                                 cacert=self._cacert,
-                                insecure=self._insecure)
+                                insecure=self._insecure,
+                                pool_maxsize=self._pool_size,
+                                connection_timeout=self._connection_timeout)
         return self._vim
 
     @property
@@ -203,7 +213,9 @@ class VMwareAPISession(object):
                                 port=self._port,
                                 wsdl_url=self._pbm_wsdl_loc,
                                 cacert=self._cacert,
-                                insecure=self._insecure)
+                                insecure=self._insecure,
+                                pool_maxsize=self._pool_size,
+                                connection_timeout=self._connection_timeout)
             if self._session_id:
                 # To handle the case where pbm property is accessed after
                 # session creation. If pbm property is accessed before session
@@ -224,7 +236,7 @@ class VMwareAPISession(object):
 
         session_manager = self.vim.service_content.sessionManager
         # Login and create new session with the server for making API calls.
-        LOG.debug("Logging in with username = %s.", self._server_username)
+        LOG.debug("Logging into host: %s.", self._host)
         session = self.vim.Login(session_manager,
                                  userName=self._server_username,
                                  password=self._server_password)
@@ -309,7 +321,7 @@ class VMwareAPISession(object):
                             {'session': _trunc_id(self._session_id),
                              'module': module,
                              'method': method})
-                        LOG.warn(excep_msg, exc_info=True)
+                        LOG.debug(excep_msg)
                         self._create_session()
                         raise exceptions.VimConnectionException(excep_msg,
                                                                 excep)
@@ -321,7 +333,9 @@ class VMwareAPISession(object):
                         LOG.debug("Fault list: %s", excep.fault_list)
                         fault = excep.fault_list[0]
                         clazz = exceptions.get_fault_class(fault)
-                        raise clazz(six.text_type(excep), excep.details)
+                        if clazz:
+                            raise clazz(six.text_type(excep),
+                                        details=excep.details)
                     raise
 
             except exceptions.VimConnectionException:
@@ -330,12 +344,11 @@ class VMwareAPISession(object):
                     # if the session has expired. Otherwise, it could be
                     # a transient issue.
                     if not self.is_current_session_active():
-                        LOG.warn(_LW("Re-creating session due to connection "
-                                     "problems while invoking method "
-                                     "%(module)s.%(method)s."),
-                                 {'module': module,
-                                  'method': method},
-                                 exc_info=True)
+                        LOG.debug("Re-creating session due to connection "
+                                  "problems while invoking method "
+                                  "%(module)s.%(method)s.",
+                                  {'module': module,
+                                   'method': method})
                         self._create_session()
 
         return _invoke_api(module, method, *args, **kwargs)
@@ -354,11 +367,11 @@ class VMwareAPISession(object):
                 self.vim.service_content.sessionManager,
                 sessionID=self._session_id,
                 userName=self._session_username)
-        except exceptions.VimException:
-            LOG.warn(_LW("Error occurred while checking whether the "
-                         "current session: %s is active."),
-                     _trunc_id(self._session_id),
-                     exc_info=True)
+        except exceptions.VimException as ex:
+            LOG.debug("Error: %(error)s occurred while checking whether the "
+                      "current session: %(session)s is active.",
+                      {'error': six.text_type(ex),
+                       'session': _trunc_id(self._session_id)})
 
         return is_active
 
@@ -413,7 +426,12 @@ class VMwareAPISession(object):
                 error_msg = six.text_type(task_info.error.localizedMessage)
                 error = task_info.error
                 name = error.fault.__class__.__name__
-                task_ex = exceptions.get_fault_class(name)(error_msg)
+                fault_class = exceptions.get_fault_class(name)
+                if fault_class:
+                    task_ex = fault_class(error_msg)
+                else:
+                    task_ex = exceptions.VimFaultException([name],
+                                                           error_msg)
                 raise task_ex
 
     def wait_for_lease_ready(self, lease):
@@ -483,8 +501,8 @@ class VMwareAPISession(object):
                                    lease,
                                    'error')
         except exceptions.VimException:
-            LOG.warn(_LW("Error occurred while reading error message for "
-                         "lease: %s."),
-                     lease,
-                     exc_info=True)
+            LOG.warning(_LW("Error occurred while reading error message for "
+                            "lease: %s."),
+                        lease,
+                        exc_info=True)
             return "Unknown"
