@@ -41,7 +41,8 @@ class Pbm(service.Service):
     """Service class that provides access to the Storage Policy API."""
 
     def __init__(self, protocol='https', host='localhost', port=443,
-                 wsdl_url=None, cacert=None, insecure=True):
+                 wsdl_url=None, cacert=None, insecure=True, pool_maxsize=10,
+                 connection_timeout=None):
         """Constructs a PBM service client object.
 
         :param protocol: http or https
@@ -52,10 +53,15 @@ class Pbm(service.Service):
                        TLS (https) server certificate.
         :param insecure: Verify HTTPS connections using system certificates,
                          used only if cacert is not specified
+        :param pool_maxsize: Maximum number of connections in http
+                             connection pool
+        :param connection_timeout: Maximum time in seconds to wait for peer to
+                                   respond.
         """
         base_url = service.Service.build_base_url(protocol, host, port)
         soap_url = base_url + '/pbm'
-        super(Pbm, self).__init__(wsdl_url, soap_url, cacert, insecure)
+        super(Pbm, self).__init__(wsdl_url, soap_url, cacert, insecure,
+                                  pool_maxsize, connection_timeout)
 
     def set_soap_cookie(self, cookie):
         """Set the specified vCenter session cookie in the SOAP header
@@ -193,7 +199,7 @@ def get_pbm_wsdl_location(vc_version):
     pbm_service_wsdl = os.path.join(curr_dir, 'wsdl', major_minor,
                                     'pbmService.wsdl')
     if not os.path.exists(pbm_service_wsdl):
-        LOG.warn(_LW("PBM WSDL file %s not found."), pbm_service_wsdl)
+        LOG.warning(_LW("PBM WSDL file %s not found."), pbm_service_wsdl)
         return
     pbm_wsdl = urlparse.urljoin('file:', urllib.pathname2url(pbm_service_wsdl))
     LOG.debug("Using PBM WSDL location: %s.", pbm_wsdl)
@@ -216,3 +222,21 @@ def get_profiles(session, vm):
 
     return session.invoke_api(pbm, 'PbmQueryAssociatedProfile',
                               profile_manager, entity=object_ref)
+
+
+def get_profiles_by_ids(session, profile_ids):
+    """Get storage profiles by IDs.
+
+    :param session: VMwareAPISession instance
+    :param profile_ids: profile IDs
+    :return: profile objects
+    """
+    profiles = []
+    if profile_ids:
+        pbm = session.pbm
+        profile_manager = pbm.service_content.profileManager
+        profiles = session.invoke_api(pbm,
+                                      'PbmRetrieveContent',
+                                      profile_manager,
+                                      profileIds=profile_ids)
+    return profiles
