@@ -56,7 +56,7 @@ class ServiceMessagePlugin(plugin.MessagePlugin):
 
         :param node: XML value node
         """
-        if node.name == 'value':
+        if node.name == 'value' or node.name == 'val':
             node.set('xsi:type', 'xsd:string')
 
     def marshalled(self, context):
@@ -116,6 +116,9 @@ class LocalFileAdapter(requests.adapters.HTTPAdapter):
 
     See http://stackoverflow.com/a/22989322
     """
+    def __init__(self, pool_maxsize=10):
+        super(LocalFileAdapter, self).__init__(pool_connections=pool_maxsize,
+                                               pool_maxsize=pool_maxsize)
 
     def _build_response_from_file(self, request):
         file_path = request.url[7:]
@@ -131,24 +134,30 @@ class LocalFileAdapter(requests.adapters.HTTPAdapter):
 
 
 class RequestsTransport(transport.Transport):
-    def __init__(self, cacert=None, insecure=True):
+    def __init__(self, cacert=None, insecure=True, pool_maxsize=10,
+                 connection_timeout=None):
         transport.Transport.__init__(self)
         # insecure flag is used only if cacert is not
         # specified.
         self.verify = cacert if cacert else not insecure
         self.session = requests.Session()
-        self.session.mount('file:///', LocalFileAdapter())
+        self.session.mount('file:///',
+                           LocalFileAdapter(pool_maxsize=pool_maxsize))
+        self.session.mount('https://', requests.adapters.HTTPAdapter(
+            pool_connections=pool_maxsize, pool_maxsize=pool_maxsize))
         self.cookiejar = self.session.cookies
+        self._connection_timeout = connection_timeout
 
     def open(self, request):
         resp = self.session.get(request.url, verify=self.verify)
-        return six.StringIO(resp.content)
+        return six.BytesIO(resp.content)
 
     def send(self, request):
         resp = self.session.post(request.url,
                                  data=request.message,
                                  headers=request.headers,
-                                 verify=self.verify)
+                                 verify=self.verify,
+                                 timeout=self._connection_timeout)
         return transport.Reply(resp.status_code, resp.headers, resp.content)
 
 
@@ -184,12 +193,16 @@ class Service(object):
     """
 
     def __init__(self, wsdl_url=None, soap_url=None,
-                 cacert=None, insecure=True):
+                 cacert=None, insecure=True, pool_maxsize=10,
+                 connection_timeout=None):
         self.wsdl_url = wsdl_url
         self.soap_url = soap_url
         LOG.debug("Creating suds client with soap_url='%s' and wsdl_url='%s'",
                   self.soap_url, self.wsdl_url)
-        transport = RequestsTransport(cacert, insecure)
+        transport = RequestsTransport(cacert=cacert,
+                                      insecure=insecure,
+                                      pool_maxsize=pool_maxsize,
+                                      connection_timeout=connection_timeout)
         self.client = client.Client(self.wsdl_url,
                                     transport=transport,
                                     location=self.soap_url,
@@ -358,3 +371,32 @@ class Service(object):
 
     def __str__(self):
         return "vSphere object"
+
+
+class SudsLogFilter(logging.Filter):
+    """Filter to mask/truncate vCenter credentials in suds logs."""
+
+    def filter(self, record):
+        if not hasattr(record.msg, 'childAtPath'):
+            return True
+
+        # Suds will log vCenter credentials if SessionManager.Login or
+        # SessionManager.SessionIsActive fails.
+        login = (record.msg.childAtPath('/Envelope/Body/Login') or
+                 record.msg.childAtPath('/Envelope/Body/SessionIsActive'))
+        if login is None:
+            return True
+
+        if login.childAtPath('userName') is not None:
+            login.childAtPath('userName').setText('***')
+        if login.childAtPath('password') is not None:  # nosec
+            login.childAtPath('password').setText('***')  # nosec
+
+        session_id = login.childAtPath('sessionID')
+        if session_id is not None:
+            session_id.setText(session_id.getText()[-5:])
+
+        return True
+
+# Set log filter to mask/truncate vCenter credentials in suds logs.
+suds.client.log.addFilter(SudsLogFilter())
