@@ -102,6 +102,7 @@ class VMwareAPISessionTest(base.TestCase):
     PORT = 443
     USERNAME = 'admin'
     PASSWORD = 'password'
+    POOL_SIZE = 15
 
     def setUp(self):
         super(VMwareAPISessionTest, self).setUp()
@@ -122,17 +123,21 @@ class VMwareAPISessionTest(base.TestCase):
                                     _create_session,
                                     port=VMwareAPISessionTest.PORT,
                                     cacert=self.cert_mock,
-                                    insecure=False)
+                                    insecure=False,
+                                    pool_size=VMwareAPISessionTest.POOL_SIZE)
 
     def test_vim(self):
         api_session = self._create_api_session(False)
         api_session.vim
-        self.VimMock.assert_called_with(protocol=api_session._scheme,
-                                        host=VMwareAPISessionTest.SERVER_IP,
-                                        port=VMwareAPISessionTest.PORT,
-                                        wsdl_url=api_session._vim_wsdl_loc,
-                                        cacert=self.cert_mock,
-                                        insecure=False)
+        self.VimMock.assert_called_with(
+            protocol=api_session._scheme,
+            host=VMwareAPISessionTest.SERVER_IP,
+            port=VMwareAPISessionTest.PORT,
+            wsdl_url=api_session._vim_wsdl_loc,
+            cacert=self.cert_mock,
+            insecure=False,
+            pool_maxsize=VMwareAPISessionTest.POOL_SIZE,
+            connection_timeout=None)
 
     @mock.patch.object(pbm, 'Pbm')
     def test_pbm(self, pbm_mock):
@@ -373,6 +378,19 @@ class VMwareAPISessionTest(base.TestCase):
             userName=api_session._session_username)
         api_session._create_session.assert_called_once_with()
 
+    def test_invoke_api_with_unknown_fault(self):
+        api_session = self._create_api_session(True)
+        fault_list = ['NotAFile']
+
+        module = mock.Mock()
+        module.api.side_effect = exceptions.VimFaultException(fault_list,
+                                                              'Not a file.')
+        ex = self.assertRaises(exceptions.VimFaultException,
+                               api_session.invoke_api,
+                               module,
+                               'api')
+        self.assertEqual(fault_list, ex.fault_list)
+
     def test_wait_for_task(self):
         api_session = self._create_api_session(True)
         task_info_list = [('queued', 0), ('running', 40), ('success', 100)]
@@ -413,7 +431,7 @@ class VMwareAPISessionTest(base.TestCase):
         api_session.invoke_api = mock.Mock(side_effect=invoke_api_side_effect)
         task = mock.Mock()
         with mock.patch.object(greenthread, 'sleep'):
-            self.assertRaises(exceptions.VMwareDriverException,
+            self.assertRaises(exceptions.VimFaultException,
                               api_session.wait_for_task,
                               task)
         api_session.invoke_api.assert_called_with(vim_util,
@@ -531,34 +549,13 @@ class VMwareAPISessionTest(base.TestCase):
 
     def test_poll_task_unknown_exception(self):
         _unknown_exceptions = {
-            'NoDiskSpace': exceptions.VMwareDriverException,
-            'RuntimeFault': exceptions.VMwareDriverException
+            'NotAFile': exceptions.VimFaultException,
+            'RuntimeFault': exceptions.VimFaultException
         }
 
         for k, v in six.iteritems(_unknown_exceptions):
             self._poll_task_well_known_exceptions(k, v)
 
-    def _create_subclass_exception(self):
-        class VimSubClass(exceptions.VMwareDriverException):
-            pass
-        return VimSubClass
-
-    def test_register_fault_class(self):
-        exc = self._create_subclass_exception()
-        exceptions.register_fault_class('ValueError', exc)
-        self.assertEqual(exc, exceptions.get_fault_class('ValueError'))
-
-    def test_register_fault_class_override(self):
-        exc = self._create_subclass_exception()
-        exceptions.register_fault_class(exceptions.ALREADY_EXISTS, exc)
-        self.assertEqual(exc,
-                         exceptions.get_fault_class(exceptions.ALREADY_EXISTS))
-
-    def test_register_fault_classi_invalid(self):
-        self.assertRaises(TypeError,
-                          exceptions.register_fault_class,
-                          'ValueError', ValueError)
-
     def test_update_pbm_wsdl_loc(self):
         session = mock.Mock()
         session.key = "12345"
