@@ -53,12 +53,12 @@ class ServiceTest(base.TestCase):
         self.SudsClientMock = patcher.start()
 
     def test_retrieve_properties_ex_fault_checker_with_empty_response(self):
-        try:
-            service.Service._retrieve_properties_ex_fault_checker(None)
-            assert False
-        except exceptions.VimFaultException as ex:
-            self.assertEqual([exceptions.NOT_AUTHENTICATED],
-                             ex.fault_list)
+        ex = self.assertRaises(
+            exceptions.VimFaultException,
+            service.Service._retrieve_properties_ex_fault_checker,
+            None)
+        self.assertEqual([exceptions.NOT_AUTHENTICATED],
+                         ex.fault_list)
 
     def test_retrieve_properties_ex_fault_checker(self):
         fault_list = ['FileFault', 'VimFault']
@@ -72,11 +72,11 @@ class ServiceTest(base.TestCase):
         response = mock.Mock()
         response.objects = [obj_cont]
 
-        try:
-            service.Service._retrieve_properties_ex_fault_checker(response)
-            assert False
-        except exceptions.VimFaultException as ex:
-            self.assertEqual(fault_list, ex.fault_list)
+        ex = self.assertRaises(
+            exceptions.VimFaultException,
+            service.Service._retrieve_properties_ex_fault_checker,
+            response)
+        self.assertEqual(fault_list, ex.fault_list)
 
     def test_request_handler(self):
         managed_object = 'VirtualMachine'
@@ -142,7 +142,7 @@ class ServiceTest(base.TestCase):
         self.assertEqual(fault_list, ex.fault_list)
         self.assertEqual({'name': 'value'}, ex.details)
         self.assertEqual("MyFault", ex.msg)
-        doc.childAtPath.assertCalledOnceWith('/detail')
+        doc.childAtPath.assert_called_once_with('/detail')
 
     def test_request_handler_with_empty_web_fault_doc(self):
 
@@ -227,7 +227,7 @@ class ServiceTest(base.TestCase):
         self.assertEqual([exceptions.NOT_AUTHENTICATED], ex.fault_list)
         self.assertEqual({'name': 'value'}, ex.details)
         self.assertEqual("MyFault", ex.msg)
-        doc.childAtPath.assertCalledOnceWith('/detail')
+        doc.childAtPath.assert_called_once_with('/detail')
 
     def test_request_handler_with_attribute_error(self):
         managed_object = 'VirtualMachine'
@@ -384,7 +384,7 @@ class MemoryCacheTest(base.TestCase):
         cache.put('key2', 'value2')
         self.assertEqual('value1', cache.get('key1'))
         self.assertEqual('value2', cache.get('key2'))
-        self.assertEqual(None, cache.get('key3'))
+        self.assertIsNone(cache.get('key3'))
 
     @mock.patch('suds.reader.DocumentReader.download')
     def test_shared_cache(self, mock_reader):
@@ -413,7 +413,7 @@ class RequestsTransportTest(base.TestCase):
     def test_open(self):
         transport = service.RequestsTransport()
 
-        data = "Hello World"
+        data = b"Hello World"
         resp = mock.Mock(content=data)
         transport.session.get = mock.Mock(return_value=resp)
 
@@ -440,6 +440,15 @@ class RequestsTransportTest(base.TestCase):
         self.assertEqual(mock.sentinel.headers, reply.headers)
         self.assertEqual(mock.sentinel.content, reply.message)
 
+    def test_set_conn_pool_size(self):
+        transport = service.RequestsTransport(pool_maxsize=100)
+        local_file_adapter = transport.session.adapters['file:///']
+        self.assertEqual(100, local_file_adapter._pool_connections)
+        self.assertEqual(100, local_file_adapter._pool_maxsize)
+        https_adapter = transport.session.adapters['https://']
+        self.assertEqual(100, https_adapter._pool_connections)
+        self.assertEqual(100, https_adapter._pool_maxsize)
+
     @mock.patch('os.path.getsize')
     def test_send_with_local_file_url(self, get_size_mock):
         transport = service.RequestsTransport()
@@ -476,3 +485,92 @@ class RequestsTransportTest(base.TestCase):
         with mock.patch(builtin_open, open_mock, create=True):
             resp = transport.session.send(request)
             self.assertEqual(data, resp.content)
+
+    def test_send_with_connection_timeout(self):
+        transport = service.RequestsTransport(connection_timeout=120)
+
+        request = mock.Mock(url=mock.sentinel.url,
+                            message=mock.sentinel.message,
+                            headers=mock.sentinel.req_headers)
+        with mock.patch.object(transport.session, "post") as mock_post:
+            transport.send(request)
+            mock_post.assert_called_once_with(
+                mock.sentinel.url,
+                data=mock.sentinel.message,
+                headers=mock.sentinel.req_headers,
+                timeout=120,
+                verify=transport.verify)
+
+
+class SudsLogFilterTest(base.TestCase):
+    """Tests for SudsLogFilter."""
+
+    def setUp(self):
+        super(SudsLogFilterTest, self).setUp()
+        self.log_filter = service.SudsLogFilter()
+
+        self.login = mock.Mock(spec=suds.sax.element.Element)
+        self.username = suds.sax.element.Element('username').setText('admin')
+        self.password = suds.sax.element.Element('password').setText(
+            'password')
+        self.session_id = suds.sax.element.Element('session_id').setText(
+            'abcdef')
+
+        def login_child_at_path_mock(path):
+            if path == 'userName':
+                return self.username
+            if path == 'password':
+                return self.password
+            if path == 'sessionID':
+                return self.session_id
+
+        self.login.childAtPath.side_effect = login_child_at_path_mock
+
+    def test_filter_with_no_child_at_path(self):
+        message = mock.Mock(spec=object)
+        record = mock.Mock(msg=message)
+        self.assertTrue(self.log_filter.filter(record))
+
+    def test_filter_with_login_failure(self):
+        message = mock.Mock(spec=suds.sax.element.Element)
+
+        def child_at_path_mock(path):
+            if path == '/Envelope/Body/Login':
+                return self.login
+
+        message.childAtPath.side_effect = child_at_path_mock
+        record = mock.Mock(msg=message)
+
+        self.assertTrue(self.log_filter.filter(record))
+        self.assertEqual('***', self.username.getText())
+        self.assertEqual('***', self.password.getText())
+        self.assertEqual('bcdef', self.session_id.getText())
+
+    def test_filter_with_session_is_active_failure(self):
+        message = mock.Mock(spec=suds.sax.element.Element)
+
+        def child_at_path_mock(path):
+            if path == '/Envelope/Body/SessionIsActive':
+                return self.login
+
+        message.childAtPath.side_effect = child_at_path_mock
+        record = mock.Mock(msg=message)
+
+        self.assertTrue(self.log_filter.filter(record))
+        self.assertEqual('***', self.username.getText())
+        self.assertEqual('***', self.password.getText())
+        self.assertEqual('bcdef', self.session_id.getText())
+
+    def test_filter_with_unknown_failure(self):
+        message = mock.Mock(spec=suds.sax.element.Element)
+
+        def child_at_path_mock(path):
+            return None
+
+        message.childAtPath.side_effect = child_at_path_mock
+        record = mock.Mock(msg=message)
+
+        self.assertTrue(self.log_filter.filter(record))
+        self.assertEqual('admin', self.username.getText())
+        self.assertEqual('password', self.password.getText())
+        self.assertEqual('abcdef', self.session_id.getText())
